use crate::{PULSE_MAX_US, PULSE_MIN_US, PULSE_NEUTRAL_US};
use log::info;
use rppal::pwm::Pwm;
use std::thread;
use std::time::Duration;
use thiserror::Error;

#[derive(Error, Debug)]
pub enum Error {
    #[error("PWM error")]
    PWMError(#[from] rppal::pwm::Error),
}

pub struct RollBot {
    pub pwm: Pwm,
    pub move_time: Duration,
}

impl RollBot {
    pub fn set_pwm_output(&self, pulse: Duration) -> Result<(), Error> {
        info!("Going to {:?}", pulse);
        self.pwm
            .set_pulse_width(pulse)?;

        Ok(())
    }

    pub fn roll(&self, tumble_times: u32) -> Result<(), Error> {
        for tumble in 1..=tumble_times {
            info!("Doing Tumble {}", tumble);

            for step in (PULSE_MIN_US..=PULSE_MAX_US).step_by(1000) {
                self.set_pwm_output(Duration::from_micros(step))?;
                thread::sleep(self.move_time);
            }

            for step in (PULSE_MIN_US..=PULSE_MAX_US).rev().step_by(1000) {
                self.set_pwm_output(Duration::from_micros(step))?;
                thread::sleep(self.move_time);
            }
        }

        info!("Roll complete!");

        Ok(())
    }
}
